@@ -32,6 +32,8 @@ __docformat__ = 'epytext en'
 __all__       = ('AbstractValueObject',)
 
 
+import sys
+
 from types           import NoneType
 
 from notify.mediator import AbstractMediator
@@ -73,7 +75,7 @@ class AbstractValueObject (object):
     _additional_description
 
     @group Internals:
-    __get_changed_signal, __to_string, __freeze_flag, __signal
+    __get_changed_signal, __to_string, __flags, __signal
 
     @sort:
     get, set, mutable, changed,
@@ -85,10 +87,19 @@ class AbstractValueObject (object):
     _additional_description
     """
 
-    __slots__ = ('__weakref__', as_string.__signal, as_string.__freeze_flag)
+    __slots__ = ('__weakref__', as_string.__signal, as_string.__flags)
 
 
-    # Implementation note: `__freeze_flag' can be None (not frozen) or True (frozen.)
+    # Implementation note: `__flags' are a sum of following values:
+    # * 0 if there is no signal, 1 if `__signal' is an `AbstractSignal' instance, 2 if it
+    #   is a reference to one;
+    # * 0 if the object is not frozen, -4 if it is.
+    #
+    # The reason for the first term is to save one (slow) isinstance() call per `changed'
+    # emission.  The second term is actually needed anyway, it is only `weird', since we
+    # need to combine with the first.
+    #
+    # We rely on Python's caching of small integers, otherwise this does waste memory.
 
 
     def __init__(self):
@@ -103,8 +114,8 @@ class AbstractValueObject (object):
         # For optimization reasons, `__signal' is created only when it is needed for the
         # first time.  This may improve memory consumption if there are many unused
         # properties.
-        self.__signal      = None
-        self.__freeze_flag = None
+        self.__signal = None
+        self.__flags  = 0
 
 
     def get (self):
@@ -163,12 +174,18 @@ class AbstractValueObject (object):
 
 
     def __get_changed_signal (self):
-        if self.__signal is not None:
-            signal = self.__signal
-            if not isinstance (signal, AbstractSignal):
-                signal = signal ()
+        flags = self.__flags
+        if flags & 3:
+            if flags & 1:
+                return self.__signal
+            else:
+                return self.__signal ()
+
+        signal, self.__signal = self._create_signal ()
+        if self.__signal is signal:
+            self.__flags += 1
         else:
-            signal, self.__signal = self._create_signal ()
+            self.__flags += 2
 
         return signal
 
@@ -230,7 +247,8 @@ class AbstractValueObject (object):
         """
 
         if self.__signal is signal:
-            self.__signal = None
+            self.__signal  = None
+            self.__flags  &= ~3
             return True
         else:
             return False
@@ -542,7 +560,7 @@ class AbstractValueObject (object):
         method I{must not} be called from outside, it is for class descendants only.
 
         To follow general contract of the class, this method must be called only when the
-        value indeed changes, i.e when C{new_value} is not equal to C{self.get()}.
+        value indeed changes, i.e. when C{new_value} is not equal to C{self.get()}.
         C{_value_changed} itself does not check it and so this check (if needed) is up to
         implementing descendant class.
 
@@ -555,13 +573,11 @@ class AbstractValueObject (object):
         @returns:         Always C{True}.
         """
 
-        if self.__freeze_flag is None:
-            signal = self.__signal
-            if signal is not None:
-                if isinstance (signal, AbstractSignal):
-                    signal.emit (new_value)
-                else:
-                    signal ().emit (new_value)
+        flags = self.__flags
+        if flags == 1:
+            self.__signal.emit (new_value)
+        elif flags == 2:
+            self.__signal ().emit (new_value)
 
         return True
 
@@ -579,7 +595,7 @@ class AbstractValueObject (object):
         @see:   C{L{changes_frozen}}
         """
 
-        return self.__freeze_flag is not None
+        return self.__flags < 0
 
 
     def with_changes_frozen (self, callback, *arguments):
@@ -620,15 +636,15 @@ class AbstractValueObject (object):
 
         # Note: keep in sync with changes_frozen() in `notify/_2_5/base.py'.
 
-        if self.__freeze_flag is None:
-            original_value     = self.get ()
-            self.__freeze_flag = True
+        if self.__flags >= 0:
+            original_value  = self.get ()
+            self.__flags   -= 4
 
             try:
                 return callback (*arguments)
             finally:
-                self.__freeze_flag = None
-                new_value          = self.get ()
+                self.__flags += 4
+                new_value     = self.get ()
 
                 if new_value != original_value:
                     self._value_changed (new_value)
@@ -782,10 +798,22 @@ class AbstractValueObject (object):
                 else:
                     dictionary['__slots__'] += tuple (value[1])
 
+        metaclass = dictionary.get ('__metaclass__', type (cls))
+        new_type  = metaclass (new_class_name, (cls,), dictionary)
+
         try:
-            return type (new_class_name, (cls,), dictionary)
-        finally:
-            del dictionary
+            raise Exception
+        except Exception:
+            try:
+                # We try to pretend that the new type is created by the caller module, not
+                # by `notify.base'.  That will give more helpful __repr__ result.
+                traceback           = sys.exc_info () [2]
+                new_type.__module__ = traceback.tb_frame.f_back.f_globals['__name__']
+            except RuntimeError:
+                # We can do nothing, ignore.
+                pass
+
+        return new_type
 
 
     def _generate_derived_type_dictionary (cls, options):
@@ -848,7 +876,9 @@ class AbstractValueObject (object):
         @raises exception: if there is any error in C{options}.
         """
 
-        functions = {}
+        functions        = {}
+        filtered_options = AbstractValueObject._filter_options (options, 'cls', 'getter', 'setter')
+
 
         if 'object' in options:
             object = options['object']
@@ -861,7 +891,7 @@ class AbstractValueObject (object):
                    '    cls.__init__(self)\n'
                    '    %s = %s')
                   % (object, AbstractValueObject._get_object (options), object)) \
-                  in functions
+                  in filtered_options, functions
 
             if 'property' in options:
                 property = options['property']
@@ -885,8 +915,6 @@ class AbstractValueObject (object):
             if not cls.__dictoffset__:
                 yield '__slots__', '__dict__'
 
-        filtered_options = AbstractValueObject._filter_options (options, 'getter', 'setter')
-
         if 'getter' in options:
             if not is_callable (options['getter']):
                 raise TypeError ("'getter' must be a callable")
