@@ -69,7 +69,7 @@ __all__       = ('Binding', 'WeakBinding', 'RaisingWeakBinding',
 from types        import FunctionType, MethodType
 import weakref
 
-from notify.utils import DummyReference
+from notify.utils import is_callable, as_string, DummyReference
 
 
 
@@ -116,8 +116,8 @@ class Binding (object):
                                  not iterable.
         """
 
-        if not callable (callable_object):
-            raise TypeError ("`callable_object' must be callable")
+        if not is_callable (callable_object):
+            raise TypeError ("'callable_object' must be callable")
 
         # This raises `TypeError' if `arguments' type is inappropriate.
         arguments = tuple (arguments)
@@ -165,8 +165,8 @@ class Binding (object):
         if arguments:
             return cls (callable_object, arguments)
         else:
-            if not callable (callable_object):
-                raise TypeError ("`callable_object' must be callable")
+            if not is_callable (callable_object):
+                raise TypeError ("'callable_object' must be callable")
 
             return callable_object
 
@@ -306,7 +306,7 @@ class Binding (object):
         if _class is not None or object is not None:
             _hash = hash (MethodType (self._get_function (), object, _class))
         else:
-            _hash = id (self._get_function ())
+            _hash = hash (self._get_function ())
 
         if arguments:
             return _hash ^ hash (arguments)
@@ -457,7 +457,7 @@ class WeakBinding (Binding):
     @see:  RaisingWeakBinding
     """
 
-    __slots__ = ('_WeakBinding__callback', '_WeakBinding__hash')
+    __slots__ = (as_string.__callback, as_string.__hash)
 
 
     def __init__(self, callable_object, arguments = (), callback = None):
@@ -494,8 +494,8 @@ class WeakBinding (Binding):
         super (WeakBinding, self).__init__(callable_object, arguments)
 
         if self._object is not None:
-            if callback is not None and not callable (callback):
-                raise TypeError ("`callback' must be callable")
+            if callback is not None and not is_callable (callback):
+                raise TypeError ("'callback' must be callable")
 
             try:
                 self.__callback = callback
@@ -586,7 +586,7 @@ class WeakBinding (Binding):
 
     def __hash__(self):
         if self.__hash is None:
-            if bool (self):
+            if self:
                 self.__hash = super (WeakBinding, self).__hash__()
             else:
                 raise TypeError (("%s's object had been garbage-collected"
