@@ -56,7 +56,7 @@ __docformat__ = 'epytext en'
 __all__       = ('AbstractMediator', 'BooleanMediator', 'FunctionalMediator')
 
 
-from notify.utils import raise_not_implemented_exception
+from notify.utils import is_callable, as_string, raise_not_implemented_exception
 
 
 
@@ -126,10 +126,10 @@ class AbstractMediator (object):
         @raises TypeError: if C{function} is not callable.
         """
 
-        if callable (function):
+        if is_callable (function):
             return _Forward (self, function)
         else:
-            raise TypeError ("`function' must be callable")
+            raise TypeError ("'function' must be callable")
 
     def back (self, function):
         """
@@ -149,10 +149,10 @@ class AbstractMediator (object):
         @raises TypeError: if C{function} is not callable.
         """
 
-        if callable (function):
+        if is_callable (function):
             return _Back (self, function)
         else:
-            raise TypeError ("`function' must be callable")
+            raise TypeError ("'function' must be callable")
 
 
     def reverse (self):
@@ -176,6 +176,7 @@ class AbstractMediator (object):
         return _ReverseMediator (self)
 
 
+    # Note: only present as a means for stating that ``mediators are comparable.''
     def __eq__(self, other):
         """
         Determine if two mediators are equal.  This function is not required to catch all
@@ -203,9 +204,6 @@ class AbstractMediator (object):
         else:
             return NotImplemented
 
-    def __hash__(self):
-        return id (self)
-
 
 
 #-- Standard mediator classes ----------------------------------------
@@ -233,16 +231,15 @@ class BooleanMediator (AbstractMediator):
         ... mediator.back_value    (15)       == 'apple'
     """
 
-    __slots__ = ('_BooleanMediator__true_value', '_BooleanMediator__false_value',
-                 '_BooleanMediator__fallback')
+    __slots__ = (as_string.__true_value, as_string.__false_value, as_string.__fallback)
 
 
     def __init__(self, true_value = True, false_value = False, fallback = None):
         if fallback is None:
             fallback = bool
         else:
-            if not callable (fallback):
-                raise TypeError ("`fallback' must be a callable")
+            if not is_callable (fallback):
+                raise TypeError ("'fallback' must be a callable")
 
         super (BooleanMediator, self).__init__()
 
@@ -287,13 +284,12 @@ class FunctionalMediator (AbstractMediator):
     (actually, anything callable.)
     """
 
-    __slots__ = ('_FunctionalMediator__forward_function', '_FunctionalMediator__back_function',
-                 '_FunctionalMediator__arguments')
+    __slots__ = (as_string.__forward_function, as_string.__back_function, as_string.__arguments)
 
 
     def __init__(self, forward_function = None, back_function = None, *arguments):
-        if (   not (forward_function is None or callable (forward_function))
-            or not (back_function    is None or callable (back_function))):
+        if (   not (forward_function is None or is_callable (forward_function))
+            or not (back_function    is None or is_callable (back_function))):
             raise TypeError ('both functions must be callable or None')
 
         super (FunctionalMediator, self).__init__()
@@ -342,7 +338,7 @@ def _identity (value, *ignored_arguments):
 
 class _ReverseMediator (AbstractMediator):
 
-    __slots__ = ('_ReverseMediator__wrapped_mediator')
+    __slots__ = (as_string.__wrapped_mediator)
 
 
     def __init__(self, wrapped_mediator):
@@ -401,7 +397,7 @@ class _Function (object):
         return not self.__eq__(other)
 
     def __hash__(self):
-        return id (self.__class__) ^ hash (self._mediator) ^ hash (self._function)
+        return hash (type (self)) ^ hash (self._mediator) ^ hash (self._function)
 
 
 
